﻿using Microsoft.EntityFrameworkCore;
using ReactiveUI;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Teachers.Models;
using Tmds.DBus.Protocol;
using MsBox.Avalonia.Enums;
using MsBox.Avalonia;
using System.Diagnostics;
using Avalonia.Controls.Documents;

namespace Teachers.ViewModels
{
    internal class ShowTeacherVM : ViewModelBase
    {
        List<Teacher> _teacherList;  

        public List<Teacher> TeacherList { get => _teacherList; set => this.RaiseAndSetIfChanged(ref _teacherList, value); } // свойство для обращения к списку с данными

        public ShowTeacherVM()
        {
            TeacherList = MainWindowViewModel.myConnection.Teachers
                                                                   .Include(x => x.Gender)
                                                                   .Include(x => x.Teachercourses).ThenInclude(x => x.Course)
                                                                   .Include(x => x.Teacherdisciplines).ThenInclude(x => x.Discipline).ToList();
        }
        public void ToAddTeacher()
        {
            MainWindowViewModel.Instance.PageContent = new OneTeacherV();
        }

        public void ToUpdateTeacher(int idTeacher)
        {
            MainWindowViewModel.Instance.PageContent = new OneTeacherV(idTeacher);
        }

        public async void DeleteTeacherAsync(int idTeacher)
        {
            ButtonResult result = await MessageBoxManager.GetMessageBoxStandard("Удаление", "Вы действительно хотите удалить этого преподователя?", ButtonEnum.YesNo).ShowAsync();

            switch (result)
            {
                case ButtonResult.Yes:
                    {
                        Teacher deleteTeacher = MainWindowViewModel.myConnection.Teachers.First(x => x.IdTeacher == idTeacher); // ищем объект в базе по переданному id
                        MainWindowViewModel.myConnection.Teachers.Remove(deleteTeacher); // удаляем объект из БД
                        MainWindowViewModel.myConnection.SaveChanges(); // сохраняем измекнения в БД
                        MainWindowViewModel.Instance.PageContent = new ShowTeacherV(); // перезагружаем страницу
                        break;
                    }
                case ButtonResult.No:
                    {                     
                        break;
                    }
            }
        }

        string _searchTeacher;

        bool _checkPhone;
        public bool CheckPhone 
        {   
            get => _checkPhone; 
            set
            {
                _checkPhone = value;
                AllFilters();
            }
        }
        public string SearchTeacher 
        {
            get => _searchTeacher;
            set
            {
                _searchTeacher = value;
                AllFilters();
            }
        }

        private List<Discipline> _disciplineFilter = [new Discipline() { IdDiscipline = 0, Title = "" }, .. MainWindowViewModel.myConnection.Disciplines.ToList()];
        public List<Discipline> DisciplineFilter { get => _disciplineFilter; }

        Discipline _selectedDisciplineFilter = null; // поле для хранения выбранной черты характера из списка для фильтрации по породам 
        public Discipline SelectedDisciplineFilter 
        {
            get
            {
                if (_selectedDisciplineFilter == null) return DisciplineFilter[0]; // для первоначальной отрисовке ссписке по умолчанию будет отображен элемент "все породы"
                else
                {
                    return _selectedDisciplineFilter;
                }
            }
            set 
            { 
                _selectedDisciplineFilter = value; AllFilters(); 
            } 
        } // свойство для обращения к полю с выбранной чертой характера

        public bool IsCheckM { get => _isCheckM; set { _isCheckM = value; AllFilters(); }
}
        public bool IsCheckW { get => _isCheckW; set { _isCheckW = value; AllFilters(); }
}
        public bool IsCheckWM { get => _isCheckWM; set { _isCheckWM = value; AllFilters(); } }

        bool _isCheckM;
        bool _isCheckW;
        bool _isCheckWM = true;

        public bool IsCheckF
        {
            get => _isCheckF; set { _isCheckF = value; AllFilters(); VisibleSort = true; Massege1 = "От Я до А"; Massege2 = "От А до Я"; SelectedSort = 0; }
        }
        public bool IsCheckE
        {
            get => _isCheckE; set { _isCheckE = value; AllFilters(); VisibleSort = true; Massege1 = "По убыванию"; Massege2 = "По возростанию"; SelectedSort = 0; }
        }
        public bool IsCheckCH
        {
            get => _isCheckCH; set { _isCheckCH = value; AllFilters(); VisibleSort = true; Massege1 = "По убыванию"; Massege2 = "По возростанию"; SelectedSort = 0; }
        }
        public bool IsCheckNo { get => _isCheckNo; set { _isCheckNo = value; AllFilters(); VisibleSort = false; } }

        bool _isCheckF;
        bool _isCheckE;
        bool _isCheckCH;
        bool _isCheckNo = true;

        bool _visibleSort = false;
        public bool VisibleSort { get => _visibleSort; set => this.RaiseAndSetIfChanged(ref _visibleSort,value); }

        string _massege1;
        string _massege2;

        // сортировка:
        int _selectedSort = 0;// поле для хранения выбранного параметра сортировки (по умолчанию будет стоять параметр "Без сортировки")
        public int SelectedSort { get => _selectedSort; set { this.RaiseAndSetIfChanged(ref _selectedSort,value); AllFilters(); } }// свойство, взаимодействующее с полем для хранения выбранного параметра сортировки

        public string Massege1 { get => _massege1; set => this.RaiseAndSetIfChanged(ref _massege1, value); }
        public string Massege2 { get => _massege2; set => this.RaiseAndSetIfChanged(ref _massege2, value); }

        void AllFilters()
        {
            TeacherList = MainWindowViewModel.myConnection.Teachers
                                                                   .Include(x => x.Gender)
                                                                   .Include(x => x.Teachercourses).ThenInclude(x => x.Course)
                                                                   .Include(x => x.Teacherdisciplines).ThenInclude(x => x.Discipline).ToList();

            // поисковая строка
            if (!string.IsNullOrWhiteSpace(_searchTeacher))
            {
                TeacherList = TeacherList.Where(x => x.FIO.ToLower().Contains(_searchTeacher.ToLower())).ToList();
            }

            // флажок номера телефонов
            if (_checkPhone)
            {
                TeacherList = TeacherList.Where(x => x.Phone == "").ToList();
            }

            // список черт характера
            if (_selectedDisciplineFilter != null && _selectedDisciplineFilter.IdDiscipline != 0)
            {
                TeacherList = TeacherList.Where(x => x.Teacherdisciplines.Any(y => y.Discipline.IdDiscipline == _selectedDisciplineFilter.IdDiscipline)).ToList();
                // Any() возвращает true, если в коллекции есть хот бы один элемент, удовлетворяющий условию
            }

            if (!_isCheckWM)
            {
                if (_isCheckM)
                {
                    TeacherList = TeacherList.Where(x => x.Gender.IdGender == 1).ToList();
                }
                else if (_isCheckW)
                {
                    TeacherList = TeacherList.Where(x => x.Gender.IdGender == 2).ToList();
                }
            }

            if (!_isCheckWM)
            {
                if (_isCheckM)
                {
                    TeacherList = TeacherList.Where(x => x.Gender.IdGender == 1).ToList();
                }
                else if (_isCheckW)
                {
                    TeacherList = TeacherList.Where(x => x.Gender.IdGender == 2).ToList();
                }
            }

            if (!_isCheckNo)
            {
                if (_isCheckF)
                {
                    switch (_selectedSort)
                    {
                        case 0:
                            break;
                        case 1:
                            TeacherList = TeacherList.OrderByDescending(x => x.Firstname).ToList();
                            break;
                        case 2:
                            TeacherList = TeacherList.OrderBy(x => x.Firstname).ToList();
                            break;
                        default:
                            break;
                    }
                }
                else if (_isCheckE)
                {
                    switch (_selectedSort)
                    {
                        case 0:
                            break;
                        case 1:
                            TeacherList = TeacherList.OrderByDescending(x => x.Experience).ToList();
                            break;
                        case 2:
                            TeacherList = TeacherList.OrderBy(x => x.Experience).ToList();
                            break;
                        default:
                            break;
                    }
                }
                else if (_isCheckCH)
                {
                    switch (_selectedSort)
                    {
                        case 0:
                            break;
                        case 1:
                            TeacherList = TeacherList.OrderByDescending(x => x.CountHour).ToList();
                            break;
                        case 2:
                            TeacherList = TeacherList.OrderBy(x => x.CountHour).ToList();
                            break;
                        default:
                            break;
                    }

                }
            }
        }
    }
}
